@@ -1,5 +1,5 @@
 -- hkt.hs: hOpenPGP key tool
--- Copyright © 2013-2014  Clint Adams
+-- Copyright © 2013-2015  Clint Adams
 --
 -- vim: softtabstop=4:shiftwidth=4:expandtab
 --
@@ -32,14 +32,16 @@ import Control.Lens ((^.), _1, _2, (^..))
 import Control.Monad.Trans.Resource (runResourceT, MonadResource)
 import qualified Control.Monad.Trans.State.Lazy as S
 import qualified Data.Aeson as A
+import Data.Binary (get, put)
+import Data.Binary.Put (runPut)
 import qualified Data.ByteString as B
 import qualified Data.ByteString.Lazy as BL
 import Data.Conduit (($=),($$), Source)
 import qualified Data.Conduit.Binary as CB
-import Data.Conduit.Cereal (conduitGet)
 import qualified Data.Conduit.List as CL
 import Data.Conduit.OpenPGP.Filter (conduitTKFilter, FilterPredicates(RTKFilterPredicate))
 import Data.Conduit.OpenPGP.Keyring (conduitToTKsDropping, sinkKeyringMap)
+import Data.Conduit.Serialization.Binary (conduitGet)
 import Data.Data.Lens (biplate)
 import Data.Either (rights)
 import qualified Data.IxSet as IxSet
@@ -56,7 +58,7 @@ import Data.Map (Map)
 import qualified Data.Map as Map
 import Data.Maybe (fromMaybe, mapMaybe, listToMaybe)
 import Data.Monoid ((<>), mconcat)
-import Data.Serialize (get, put, runPut)
+import Data.Text (Text)
 import qualified Data.Text as T
 import qualified Data.Text.Lazy.IO as TLIO
 import Data.Time.Clock.POSIX (getPOSIXTime, posixSecondsToUTCTime)
@@ -73,19 +75,19 @@ import Options.Applicative.Types (Parser)
 import System.IO (Handle, hFlush, stderr, hSetBuffering, BufferMode(..))
 import Text.PrettyPrint.ANSI.Leijen ((<+>), (</>), hardline, hPutDoc, list, putDoc, text)
 
-grabMatchingKeysConduit :: MonadResource m => FilePath -> Bool -> String -> Source m TK
+grabMatchingKeysConduit :: MonadResource m => FilePath -> Bool -> Text -> Source m TK
 grabMatchingKeysConduit fp filt srch = CB.sourceFile fp $= conduitGet get $= conduitToTKsDropping $= (if filt then conduitTKFilter ufp else CL.filter matchAny)
     where
         matchAny tk = either (const False) id $ fmap (keyMatchesFingerprint True tk) efp <|> fmap (keyMatchesEightOctetKeyId True tk . Right) eeok <|> return (keyMatchesUIDSubString srch tk)
-        efp = parseFingerprint . T.pack $ srch
-        eeok = parseEightOctetKeyId . T.pack $ srch
+        efp = parseFingerprint srch
+        eeok = parseEightOctetKeyId srch
         ufp = RTKFilterPredicate (parseE srch)
-        parseE e = either (error . ("filter parse error: "++)) id (parseTKExp e) -- this should be more specialized
+        parseE = either (error . ("filter parse error: " ++)) id . parseTKExp . T.unpack -- this should be more specialized
 
-grabMatchingKeys :: FilePath -> Bool -> String -> IO [TK]
+grabMatchingKeys :: FilePath -> Bool -> Text -> IO [TK]
 grabMatchingKeys fp filt srch = runResourceT $ grabMatchingKeysConduit fp filt srch $$ CL.consume
 
-grabMatchingKeysKeyring :: FilePath -> Bool -> String -> IO Keyring
+grabMatchingKeysKeyring :: FilePath -> Bool -> Text -> IO Keyring
 grabMatchingKeysKeyring fp filt srch = runResourceT $ grabMatchingKeysConduit fp filt srch $$ sinkKeyringMap
 
 data Key = Key {
@@ -98,7 +100,7 @@ data Key = Key {
 
 data TKey = TKey {
     publickey :: Key
-  , uids :: [String]
+  , uids :: [Text]
   , subkeys :: [Key]
 } deriving Generic
 
@@ -121,7 +123,7 @@ tkToTKey tk = TKey {
 showTKey :: TKey -> IO ()
 showTKey key = putDoc $
     text "pub  " <+> sizeabbrevkeyid (publickey key) <> hardline <>
-    mconcat (map (\x -> text "uid                           " <+> text x <> hardline) (uids key)) <>
+    mconcat (map (\x -> text "uid                           " <+> text (T.unpack x) <> hardline) (uids key)) <>
     mconcat (map (\x -> text "sub  " <+> sizeabbrevkeyid x <> hardline) (subkeys key)) <>
     hardline
     where
@@ -240,7 +242,8 @@ banner' h = hPutDoc h (banner "hkt" <> hardline <> warranty "hkt" <> hardline)
 
 doList :: Options -> IO ()
 doList o = do
-    keys' <- grabMatchingKeys (keyring o) (targetIsFilter o) (target1 o)
+    let ttarget1 = T.pack . target1
+    keys' <- grabMatchingKeys (keyring o) (targetIsFilter o) (ttarget1 o)
     let keys = map tkToTKey keys'
     case pathsOutputFormat o of
         Unstructured -> mapM_ showTKey keys
@@ -250,8 +253,9 @@ doList o = do
 
 doExportPubkeys :: Options -> IO ()
 doExportPubkeys o = do
-    keys <- grabMatchingKeys (keyring o) (targetIsFilter o) (target1 o)
-    mapM_ (B.putStr . putTK') keys
+    let ttarget1 = T.pack . target1
+    keys <- grabMatchingKeys (keyring o) (targetIsFilter o) (ttarget1 o)
+    mapM_ (BL.putStr . putTK') keys
     where
         putTK' key = runPut $ do
             put (PublicKey (key^.tkKey._1))
@@ -265,8 +269,9 @@ doExportPubkeys o = do
 
 doGraph :: Options -> IO ()
 doGraph o = do
+    let ttarget1 = T.pack . target1
     cpt <- getPOSIXTime
-    kr <- grabMatchingKeysKeyring (keyring o) (targetIsFilter o) (target1 o)
+    kr <- grabMatchingKeysKeyring (keyring o) (targetIsFilter o) (ttarget1 o)
     let g = buildKeyGraph ((buildMaps &&& id) (rights (map (verifyTKWith (verifySigWith (verifyAgainstKeyring kr)) (Just (posixSecondsToUTCTime cpt))) (IxSet.toList kr))))
     case graphOutputFormat o of
         LossyPretty -> prettyPrint g
@@ -313,16 +318,17 @@ data PaF = PaF {
 } deriving Generic
 
 instance A.ToJSON PaF
-instance A.ToJSON TwentyOctetFingerprint where
-    toJSON = A.toJSON . show
 
 doFindPaths :: Options -> IO ()
 doFindPaths o = do
+    let ttarget1 = T.pack . target1
+        ttarget2 = T.pack . target2
+        ttarget3 = T.pack . target3
     cpt <- getPOSIXTime
-    kr <- grabMatchingKeysKeyring (keyring o) (targetIsFilter o) (target1 o)
+    kr <- grabMatchingKeysKeyring (keyring o) (targetIsFilter o) (ttarget1 o)
     -- FIXME: clean this up
-    keys1 <- runResourceT $ CL.sourceList (IxSet.toList kr) $= (if filt then conduitTKFilter (ufpt (target2 o)) else CL.filter (matchAny (target2 o))) $$ CL.consume
-    keys2 <- runResourceT $ CL.sourceList (IxSet.toList kr) $= (if filt then conduitTKFilter (ufpt (target3 o)) else CL.filter (matchAny (target3 o))) $$ CL.consume
+    keys1 <- runResourceT $ CL.sourceList (IxSet.toList kr) $= (if filt then conduitTKFilter (ufpt (ttarget2 o)) else CL.filter (matchAny (ttarget2 o))) $$ CL.consume
+    keys2 <- runResourceT $ CL.sourceList (IxSet.toList kr) $= (if filt then conduitTKFilter (ufpt (ttarget3 o)) else CL.filter (matchAny (ttarget3 o))) $$ CL.consume
     let ((KeyMaps k2f f2i i2f, i), ks) = (buildMaps &&& id) (rights (map (verifyTKWith (verifySigWith (verifyAgainstKeyring kr)) (Just (posixSecondsToUTCTime cpt))) (IxSet.toList kr)))
         keygraph = buildKeyGraph ((KeyMaps k2f f2i i2f, i), ks)
         keysToIs = mapMaybe (\x -> HashMap.lookup (fingerprint (x^.tkKey._1)) f2i)
@@ -341,10 +347,10 @@ doFindPaths o = do
     where  -- FIXME: deduplicate this
         filt = targetIsFilter o
         matchAny srch tk = either (const False) id $ fmap (keyMatchesFingerprint True tk) (efp srch) <|> fmap (keyMatchesEightOctetKeyId True tk . Right) (eeok srch) <|> return (keyMatchesUIDSubString srch tk)
-        efp srch = parseFingerprint . T.pack $ srch
-        eeok srch = parseEightOctetKeyId . T.pack $ srch
+        efp srch = parseFingerprint srch
+        eeok srch = parseEightOctetKeyId srch
         ufpt srch = RTKFilterPredicate (parseE srch)
-        parseE e = either (error . ("filter parse error: "++)) id (parseTKExp e) -- this should be more specialized
+        parseE e = either (error . ("filter parse error: "++)) id (parseTKExp (T.unpack e)) -- this should be more specialized
 
 -- FIXME: deduplicate the following code
 sigissuer :: SignaturePayload -> Maybe EightOctetKeyId
